#!/usr/bin/env python
# coding=UTF-8

import  sys
import  re
import  pdb

import  config, genyear, photodb

from  os.path   import  basename, dirname, splitext, join, exists
from  glob      import  glob
from  codecs    import  open
from  datetime  import  datetime
from  calendar  import  monthrange, monthcalendar
from  datetool  import  date_str

#
# return correct css class for given image
#
def css_class ( img ) :
    cssclass = 'thumb'
    if   img.orientation == 'landscape' : cssclass = cssclass + 'l'
    elif img.orientation == 'portrait'  : cssclass = cssclass + 'p'
    
    cssclass = cssclass + img.format

    return cssclass

#
# write html header to file with title
#
def write_html_header ( file, css, title ) :

    file.write( '<?xml version="1.0"?>\n' )
    file.write( '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"\n' )
    file.write( '    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">\n\n' )

    file.write( '<html xmlns="http://www.w3.org/1999/xhtml"\n' )
    file.write( '      xml:lang="en">\n' )
    file.write( '<head>\n' )
    file.write( '  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />\n' )
    file.write( '  <title>' + title + '</title>\n' )
    file.write( '  <style type="text/css" media="screen">\n' )
    file.write( '    @import url(' + css + ');\n' )
    file.write( '  </style>\n' )
    file.write( '</head>\n' )
    file.write( '<body>\n' )

#
# write html footer to file
#
def write_html_footer ( file ) :

    file.write( '    <div id="footer">\n' )
    file.write( '      <div id="w3check">\n' )
    file.write( '        Darstellbar mit <a href="http://validator.w3.org/check/referer">jedem</a> Browser\n' )
    file.write( '      </div>\n' )
    file.write( '    </div>\n' )

    file.write( '  </div>\n' )
    file.write( '</body>\n' )
    file.write( '</html>\n' )

#
# write file for a day
#
def write_day ( dir, filename, album ):
    "Write index.html file containing list of all photos and videos"

    index = open( filename, encoding='utf-8', mode='w' )

    if album.title != '':
        write_html_header( index, '../../css/style.css', album.title )
    elif album.date != '':
        write_html_header( index, '../../css/style.css', 'Fotoalbum : ' + date_str( album.date ) )
    else:
        write_html_header( index, '../../css/style.css', 'Fotoalbum' )

    # toolbar
    index.write( '  <div class="toolbar">\n' )
    index.write( '    <div class="toolbtn"><a href="../../index.html" class="homebtn"><br />Start</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="../../suche/index.html" class="searchbtn"><br />Suche</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="../../hilfe/hilfe_tag.html" class="helpbtn"><br />Hilfe</a></div>\n' )
    index.write( '  </div>\n' )

    # header
    index.write( '  <div id="page">\n' )
    index.write( '    <div id="header">\n' )

    #   title
    index.write( '      <div id="title">\n' )

    if album.title != '':
        index.write( '        <h1>' + album.title + '</h1>\n' )
    else:
        index.write( '        <h1> &nbsp; </h1>\n' )

    if album.date != '':
        index.write( '        <h2>' + date_str( album.date ) + '</h2>\n' )
    else:
        index.write( '        <h2> &nbsp; </h2>\n' )
    index.write( '      </div>\n' )

    #   navigation
    index.write( '      <div id="navigation">\n' )

    if album.prev != '':
        index.write( '<a href="../' + album.prev + '/index.html"><span id="navprev"></span></a>' )
    else:
        index.write( '<span id="navprev-invalid"></span>' )

    index.write( '<a href="../index.html"><span id="navhome"></span></a>' )

    if album.next != '':
        index.write( '<a href="../' + album.next + '/index.html"><span id="navnext"></span></a>' )
    else:
        index.write( '<span id="navnext-invalid"></span>' )

    index.write( '\n      </div>\n' )

    # end of header
    index.write( '    </div>\n' )   

    # image/video list
    index.write( '    <div id="content">\n' )

    for img in album.db:
        
        index.write( '      <div class="%s">\n' % ( css_class( img ) ) )

        name = splitext( basename( img.file ) )[0]

        index.write( '        <a href="%s"><img src="%s" alt="%s" /></a>\n' %
                     ( name + '.html', img.thumb_file(), name ))

        if img.desc != '':
            index.write( '        <p>' + img.desc + '</p>\n' )

        index.write( '      </div>\n' )

    index.write( '    </div>\n' )   

    # footer
    write_html_footer( index )

    index.close()

#
# write html file for specific photo
#
def write_photo ( dir, filename, img, album, prev, next ):
    "Write html file for a specific photo"

    index = open( filename, encoding='utf-8', mode='w' )

    if album.title != '':
        write_html_header( index, '../../css/style.css', album.title )
    elif album.date != '':
        write_html_header( index, '../../css/style.css', 'Fotoalbum : ' + date_str( album.date ) )
    else:
        write_html_header( index, '../../css/style.css', 'Fotoalbum' )

    # toolbar
    index.write( '  <div class="toolbar">\n' )
    index.write( '    <div class="toolbtn"><a href="../../index.html" class="homebtn"><br />Start</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="../../suche/index.html" class="searchbtn"><br />Suche</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="../../hilfe/hilfe_foto.html" class="helpbtn"><br />Hilfe</a></div>\n' )
    index.write( '  </div>\n' )

    index.write( '  <div id="page">\n' )

    # header
    index.write( '    <div id="header">\n' )

    #   title 
    index.write( '      <div id="title">\n' )
    if album.title != '':
        index.write( '        <h1>' + album.title + '</h1>\n' )
    else:
        index.write( '        <h1>&nbsp;</h1>\n' )
    if album.date != '':
        index.write( '        <h2>' + date_str( album.date ) + '</h2>\n' )
    else:
        index.write( '        <h2>&nbsp;</h2>\n' )
    index.write( '      </div>\n' )

    #   navigation
    index.write( '      <div id="navigation">' )
    if prev != None:
        index.write( '<a href="' + prev.content_file() + '"><span id="navprev"></span></a>' )
    else:
        index.write( '<span id="navprev-invalid"></span>' )
    index.write( '<a href="index.html"><span id="navhome"></span></a>' )
    if next != None:
        index.write( '<a href="' + next.content_file() + '"><span id="navnext"></span></a>' )
    else:
        index.write( '<span id="navnext-invalid"></span>' )
    index.write( '</div>\n' )

    # end of header
    index.write( '    </div>\n' )

    # content
    index.write( '    <div id="content">\n' )

    if img.type == 'photo':
        index.write( '      <div class="photo">\n' )
        imgname = img.lores_file()
        index.write( '        <img src="' + imgname + '" alt="' + img.content_file() + '" />\n' )
        if img.desc != '' :
            index.write( '        <p>' + img.desc + '</p>\n' )
        index.write( '      </div>\n' )
        index.write( '    </div>\n' )

    elif img.type == 'video':
        index.write( '      <div class="video">\n' )
        imgname = img.lores_file()

        if img.format == '16x9':
            index.write( '        <embed width="848px" height="480px" type="video/mp4"' )
        elif img.format == '4x3':
            index.write( '        <embed width="640px" height="480px" type="video/mp4"' )
        index.write( ' src="%s"></embed>\n' % imgname )
        # index.write( '<div class="videolink"><a href="%s">herunterladen</a></div>' % imgname )
        index.write( '<div class="videolink">' )
        index.write( '  <form action="%s" target="_blank">' % imgname )
        index.write( '  <input type="submit" value="herunterladen">\n' )
        index.write( '  </form>\n' )
        index.write( '</div>' )
        if img.desc != '' :
            index.write( '        <p>' + img.desc + '</p>\n' )
        index.write( '      </div>\n' )
        index.write( '    </div>\n' )

    # footer
    write_html_footer( index )

    index.close()

#
# write xml file containing year index structure
#
def write_year ( html, year, daydb ):
    "write html file containing calendar for a year and links to existing day entries"

    index = open( html, encoding='utf-8', mode='w' )

    write_html_header( index, '../css/style.css', 'Fotoalbum' )

    # toolbar
    index.write( '  <div class="toolbar">\n' )
    index.write( '    <div class="toolbtn"><a href="../index.html" class="homebtn"><br />Start</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="../suche/index.html" class="searchbtn"><br />Suche</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="../hilfe/hilfe_jahr.html" class="helpbtn"><br />Hilfe</a></div>\n' )
    index.write( '  </div>\n' )

    index.write( '  <div id="page">\n' )

    # header
    index.write( '    <div id="header">\n' )

    #   title
    index.write( '      <div id="title">\n' )
    index.write( '        <h1>Fotoalbum</h1>\n' )
    index.write( '        <h2>' + str( year ) + '</h2>\n' )
    index.write( '      </div>\n' )
    
    #   navigation
    index.write( '      <div id="navigation">\n' )
    prev_year = genyear.prev_year( year )
    if prev_year != year and exists( join( config.top_dir, str(prev_year), 'index.html' ) ):
        index.write( '<a href="../' + str( prev_year ) + '/index.html"><span id="navprev"></span></a>' )
    else:
        index.write( '<span id="navprev-invalid"></span>' )
    index.write( '<a href="../index.html"><span id="navhome"></span></a>' )
    next_year = genyear.next_year( year )
    if next_year != year and exists( join( config.top_dir, str(next_year), 'index.html' ) ):
        index.write( '<a href="../' + str( next_year ) + '/index.html"><span id="navnext"></span></a>' )
    else:
        index.write( '<span id="navnext-invalid"></span>' )
    index.write( '</div>\n' )

    # end of header
    index.write( '    </div>\n' )

    # calendar table
    index.write( '    <div id="content">\n' )
    index.write( '      <table class="year" summary="year" cellspacing="0" cellpadding="0">\n' )
    index.write( '        <tr><td class="yearname" colspan="3">' + str( year ) + '</td></tr>\n' )
    index.write( '        <tr>\n' )

    # write months of the year, e.g. actual calendar
    for month in range(1,13):
        index.write( '          <td>\n' )
        index.write( '            <table class="month" summary="month" cellspacing="0" cellpadding="0">\n' )
        index.write( '              <tr><td class="monthname" colspan="7">' + config.monthnames[ month ] + '</td></tr>\n' )
        index.write( '              <tr>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[0] + '</td>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[1] + '</td>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[2] + '</td>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[3] + '</td>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[4] + '</td>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[5] + '</td>\n' )
        index.write( '                <td class="weekday">' + config.weekdayname[6] + '</td>\n' )
        index.write( '              </tr>\n' )

        for week in monthcalendar( year, month ):
            index.write( '              <tr class="week">\n' )
            for day in week:
                if day == 0:
                    index.write( '                <td class="noday"></td>\n' )
                else:
                    index.write( '                <td class="day">' )
                    if day in daydb[ month ]:
                        subdir = '%02d-%02d' % ( month, day )
                        db     = photodb.Album( join( config.top_dir, str( year ), subdir, 'PHOTOS.xml' ) )
                        link   = subdir + '/index.html'
                        if db.title != '':
                            index.write( '<p>' + db.title + '</p>' )
                        index.write( '<a href="' + link + '">' + str( day ) + '</a>' )
                    elif 'all' in daydb[ month ]:
                        subdir = '%02d' % ( month )
                        db     = photodb.Album( join( config.top_dir, str( year ), subdir, 'PHOTOS.xml' ) )
                        link   = subdir + '/index.html'
                        if db.title != '':
                            index.write( '<p>' + db.title + '</p>' )
                        index.write( '<a href="' + link + '">' + str( day ) + '</a>' )
                    else:
                        index.write( '<span>' + str( day ) + '</span>' )
                    index.write( '</td>\n' )                        
            index.write( '              </tr>\n' )

        index.write( '            </table>\n' )
        index.write( '          </td>\n' )

        # end of current month line in table
        if month % 3 == 0 and month != 12:
            index.write( '        </tr>\n' )
            index.write( '        <tr>\n' )
        
    index.write( '        </tr>\n' )
    index.write( '      </table>\n' )
    index.write( '    </div>\n' )

    # footer
    write_html_footer( index )

    index.close()

#
# write html file containing global index structure
#
def write_index () :
    "write html file containing global index structure"

    index = open( join( config.top_dir, 'index.html' ), encoding='utf-8', mode='w' )

    write_html_header( index, 'css/style.css', 'Fotoalbum' )

    # toolbar
    index.write( '  <div class="toolbar">\n' )
    index.write( '    <div class="toolbtn"><a href="index.html" class="homebtn"><br />Start</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="suche/index.html" class="searchbtn"><br />Suche</a></div>\n' )
    index.write( '    <div class="toolbtn"><a href="hilfe/hilfe_jahr.html" class="helpbtn"><br />Hilfe</a></div>\n' )
    index.write( '  </div>\n' )

    index.write( '  <div id="page">\n' )

    # header
    index.write( '    <div id="header">\n' )

    #   title
    index.write( '      <div id="title">\n' )
    index.write( '        <h1>Fotoalbum</h1>\n' )
    index.write( '        <h2></h2>\n' )
    index.write( '      </div>\n' )
    
    #   navigation
    index.write( '      <div id="navigation"></div>\n' )

    # end of header
    index.write( '    </div>\n' )

    # list of years
    index.write( '    <div id="content">\n' )

    #
    # determine list of years (subdirs) 
    #

    subdirs = glob( join( config.top_dir, '[12][0-9]*' ) )
    years   = []

    for dir in subdirs :
        years.append( int( basename( dir ) ) )

    # sort years in groups of three, with reversed order in each group
    years      = sorted( years )
    sort_years = []
    loc_years  = []

    last_year  = 0
    counter    = 1

    for year in years :

        if last_year != 0 and abs( year - last_year ) > 1 :
            sort_years = sort_years + loc_years
            sort_years.append( -2 )
            loc_years  = []

            if counter > 3 :
                counter = 1

        elif counter > 3 :
            sort_years = sort_years + loc_years
            sort_years.append( -1 )
            loc_years  = []
            counter = 1
            
        loc_years.insert( 0, year )
        last_year = year
        counter   = counter+1

    sort_years = sort_years + loc_years
    sort_years.reverse()
    # print( sort_years )

    # sys.exit( 0 )

    #
    # print list of years (subdirs)
    # 

    index.write( '      <table style="margin: 0 auto;">\n' )
    index.write( '        <tr>\n' )

    pos = 1

    for year in sort_years :

        if year == -1 :
            for i in range( pos, 4 ) :
                index.write( '          <td> </td>\n' )
            index.write( '        </tr>\n' )
            index.write( '        <tr>\n' )
            pos = 0

        elif year == -2 :
            for i in range( pos, 4 ) :
                index.write( '          <td> </td>\n' )

            index.write( '        </tr>\n' )
            index.write( '        <tr> <td colspan="3"> <hr style="border: 2px solid #d3d7cf;" /> </td> </tr>\n' )
            index.write( '        <tr>\n' )
            pos = 0
            
        else :
            if pos > 3 :
                index.write( '        </tr>\n' )
                index.write( '        <tr>\n' )
                pos = 0
            
            index.write( '          <td> <a class="photolink" href="%s/index.html">%s</a> </td>\n' % 
                         ( year, year ) )

        pos = pos + 1

    index.write( '        </tr>\n' )
    index.write( '      </table>\n' )
    
    # finish content
    index.write( '    </div>\n' )

    # footer
    write_html_footer( index )

    index.close()

#
# write tag html file with list of pages containing tag
#
def write_tag ( html, tag, list ):
    "write tag html file with list of pages containing tag"

    # sort list according to dates (HACK, works due to directory names! Really sort by looking at dates!)
    datelist = sorted( list, reverse = True )

    index = open( html, encoding='utf-8', mode='w' )

    write_html_header( index, '../css/style.css', 'Fotoalbum' )

    # header
    index.write( '  <div id="page">\n' )
    index.write( '    <div id="header">\n' )
    index.write( '      <div id="title">\n' )
    index.write( '        <h1>' + tag + '</h1>\n' )
    index.write( '        <h2></h2>\n' )
    index.write( '      </div>\n' )
    index.write( '      <div id="navigation">\n' )
    index.write( '        <a href="javascript: history.back()"><span id="navprev"></span></a>\n' )
    index.write( '      </div>\n' )
    index.write( '    </div>\n' )

    index.write( '      <div id="content">\n' )
    
    index.write( '        <ul class="datelist">\n' )

    is_odd = True

    for entry in datelist :
        
        idxfile = entry['db']
        dir     = dirname( idxfile )
        db      = photodb.Album( idxfile )

        # if no sub entries or more than one sub entry, start with index file
        # else, directly go to photo/video
        link = db.index_link()
        #if len( entry['entries'] ) != 1 :
        #    link = db.index_file()
        #else :
        #    link = entry['entries'][0].content_file( db.directory() )

        if is_odd : 
            index.write( '          <li class="odd">\n' )
            is_odd = False
        else :
            index.write( '          <li class="even">\n' )
            is_odd = True

        index.write( '            <a href="' + join( '..', link ) + '">' + date_str( db.date ) + '</a>\n' )
        if db.title != '' :
            index.write( '            <span class="title">' + db.title + '</span>\n' )

        # print individual photo/video entries, but only 
        # if not exceeding 10 entries
        if len( entry['entries'] ) > 0 and len( entry['entries'] ) < 10:
            basedir = db.directory( relative = True )
            index.write( '            <div class="tinythumblist">\n' )

            for subentry in entry['entries'] :
                link = join( '..', subentry.content_file( basedir ) )
                name = basename( subentry.file )
                index.write( '<div class="%s">' % ( 'tiny' + css_class( subentry ) ) )
                index.write( '<a href="' + link + '" >' )
                index.write( '<img src="%s" alt="%s" />' % ( join( '..', subentry.thumb_file( basedir ) ), name ) )
                index.write( '</a>' )
                index.write( '</div>\n' )

            index.write( '            </div>\n' )
            

        index.write( '          </li>\n' )

    index.write( '        </ul>\n' )
    index.write( '      </div>\n' )

    # footer
    write_html_footer( index )

    index.close()

#
# determine size of tag, e.g. number of entries
#
def tagsize ( tag ) :
    return len( tag )

#
# write file with tag list
#
def write_tag_list ( html, tag_db ):
    "write html file containing search form and tags"

    # upper limit to reduce influence of single large entries
    COUNT_LIMIT = 40

    # def caseinsensitive_cmp ( str1, str2 ) :
    #     if   str1.lower() <  str2.lower() : return -1
    #     elif str1.lower() == str2.lower() : return 0
    #     else                              : return 1

    # sort keys
    keys = sorted( tag_db )

    # determine minimum and maximum of tag size for font sizes
    min_n = sys.maxint
    max_n = 0
    for tag in keys :
        n     = tagsize( tag_db[ tag ] )
        min_n = min( min_n, n )
        max_n = max( max_n, n )

    max_n = min( max_n, COUNT_LIMIT ) # limit maximum

    # write html file
    index = open( html, encoding='utf-8', mode='w' )

    write_html_header( index, '../css/style.css', 'Fotoalbum' )

    # header
    index.write( '  <div id="page">\n' )
    index.write( '    <div id="header">\n' )
    index.write( '      <div id="title">\n' )
    index.write( '        <h1>Suche</h1>\n' )
    index.write( '        <h2></h2>\n' )
    index.write( '      </div>\n' )
    index.write( '      <div id="navigation">\n' )
    index.write( '        <a href="javascript: history.back()"><span id="navprev"></span></a>\n' )
    index.write( '      </div>\n' )
    index.write( '    </div>\n' )

    index.write( '    <div id="content">\n' )

    index.write( '      <div class="search">\n' )

    # index.write( '        <h1>Suche</h1>\n' )

    index.write( '        <!-- Suchformular -->\n' )
    index.write( '        <form action="http://loki.mis.mpg.de/fotos/cgi-bin/search.cgi" method="get">\n' )
    index.write( '          <table>\n' )
    index.write( '            <tr> <td> <input class="entry" name="suchtext" type="text" size="50" maxlength="200"> </td> </tr>\n' )
    index.write( '            <tr> <td> <input class="button" type="submit" value="Suchen"> </td> </tr>\n' )
    index.write( '          </table>\n' )
    index.write( '        </form>\n' )

    # index.write( '        <hr />\n' )

    index.write( '        <!-- Suchformular -->\n' )
    # index.write( '        <h2>Schlagworte</h2>\n' )
    index.write( '        <p class="tags">\n' )

    # print tags
    for tag in keys :
        
        n = tagsize( tag_db[ tag ] )

        n        = min( n, COUNT_LIMIT )
        size     = (float(n - min_n) / float(max_n - min_n)) * 300.0 + 66.0
        size_str = "%d%%" % size

        taghtml = join( config.top_dir, 'suche', 'tag_' + tag + '.html' )
        taglink = 'tag_' + tag + '.html'
        index.write( '        <a href="' + taglink + '" style="font-size: ' + size_str + ';">' + tag + '</a>\n' )
        write_tag( taghtml, tag, tag_db[ tag ] )

    index.write( '          </p>\n' )
    index.write( '      </div>\n' )
    index.write( '    </div>\n' )

    # footer
    write_html_footer( index )

    index.close()

#
# write tag html file with list of pages containing tag
#
def write_search_results ( html, searchstr, search_db, idxlist ) :
    "write tag html file with search results for given search str"

    #
    # start writing file
    #
    index = open( html, encoding='utf-8', mode='w' )

    write_html_header( index, '../css/style.css', 'Fotoalbum' )

    # header
    index.write( '  <div id="page">\n' )
    index.write( '    <div id="header">\n' )
    index.write( '      <div id="title">\n' )
    index.write( '        <h1>' + searchstr + '</h1>\n' )
    index.write( '        <h2></h2>\n' )
    index.write( '      </div>\n' )
    index.write( '      <div id="navigation">\n' )
    index.write( '        <a href="javascript: history.back()"><span id="navprev"></span></a>\n' )
    index.write( '      </div>\n' )
    index.write( '    </div>\n' )

    index.write( '      <div id="content">\n' )
    
    index.write( '        <ul class="datelist">\n' )

    is_odd = True

    for idxfile in idxlist :

        album  = photodb.Album( idxfile )
        photos = search_db[ idxfile ]

        # if no sub entries or more than one sub entry, start with index file
        # else, directly go to photo/video
        link = album.index_link()

        if is_odd : 
            index.write( '          <li class="odd">\n' )
            is_odd = False
        else :
            index.write( '          <li class="even">\n' )
            is_odd = True

        index.write( '            <a href="' + join( '..', link ) + '">' + date_str( album.date ) + '</a>\n' )
        if album.title != '' :
            index.write( '            <span class="title">' + album.title + '</span>\n' )

        # print individual photo/video entries, but only 
        # if not exceeding 10 entries
        if len( photos ) > 0 and len( photos ) < 10:
            basedir = album.directory( relative = True )
            index.write( '            <div class="tinythumblist">\n' )

            for photo in photos :
                entry = album.get_entry( photo )
                link = join( '..', entry.content_file( basedir ) )
                name = basename( entry.file )
                index.write( '<div class="%s">' % ( 'tiny' + css_class( entry ) ) )
                index.write( '<a href="' + link + '" >' )
                index.write( '<img src="%s" alt="%s" />' % ( join( '..', entry.thumb_file( basedir ) ), name ) )
                index.write( '</a>' )
                index.write( '</div>\n' )

            index.write( '            </div>\n' )
            

        index.write( '          </li>\n' )

    index.write( '        </ul>\n' )
    index.write( '      </div>\n' )

    # footer
    write_html_footer( index )

    index.close()


