#!/usr/bin/env python
# coding=UTF-8

import libxml2
import libxslt

import config

from  os.path  import  basename;

#
# return relative path of stylesheet w.r.t. given directory
#        
def stylepath ( dir, stylesheet ):
    # simply return absolute path
    return join( config.style_dir, stylesheet )

#
# convert xml to html using internal stylesheets
#
def xml_to_html ( xml, html ):
    xmlfile  = libxml2.parseFile( xml )
    style    = libxslt.loadStylesheetPI( xmlfile )
    result   = style.applyStylesheet( xmlfile, None )
    style.saveResultToFilename( html, result, 0 )
    style.freeStylesheet()
    xmlfile.freeDoc()
    result.freeDoc()

#
# write index.xml file for a day
#
def write_day ( dir, filename, imgdb, album_info ):
    "Write index.xml file containing list of all photos and videos"

    index_xml = open( filename, encoding='utf-8', mode='w' )
    index_xml.write( '<?xml version="1.0" encoding="UTF-8"?>\n' )
    index_xml.write( '<?xml-stylesheet type="text/xsl" href="' + 
                     stylepath( dir, 'album.xsl' ) + '" ?>\n' )
    index_xml.write( '<album>\n' )

    if album_info['title'] != '': 
        index_xml.write( '  <title>'   + album_info['title'] + '</title>\n' )
    if album_info['date']  != '':
        index_xml.write( '  <date>'    + album_info['date']  + '</date>\n' )
    if album_info['prev']  != '': 
        index_xml.write( '  <navprev>' + album_info['prev']  + '</navprev>\n' )
    if album_info['next']  != '': 
        index_xml.write( '  <navnext>' + album_info['next']  + '</navnext>\n' )

    for img in imgdb:
        index_xml.write( '  <thumbnail orientation="' + img['orientation'] + '"' + 
                         ' format="' + img['format'] + '">\n' )

        name = splitext( basename( img['img'] ) )[0]

        index_xml.write( '    <url>'+ name + '.xml</url>\n' )
        index_xml.write( '    <img>' + config.thumb( img ) + '</img>\n' )
        index_xml.write( '    <name>'+ name + '</name>\n' )
        if img['desc'] != '':
            index_xml.write( '    <desc>' + unicode( img['desc'] ) + '</desc>\n' )
        index_xml.write( '  </thumbnail>\n' )

    index_xml.write(  "</album>\n" )
    index_xml.close()

#
# write xml file for specific photo
#
def write_photo ( dir, filename, img, album_info, prev, next ):
    "Write xml file for a specific photo"

    xmlfile = open( filename, encoding='utf-8', mode='w' )

    xmlfile.write( '<?xml version="1.0" encoding="UTF-8"?>\n' )
    xmlfile.write( '<?xml-stylesheet type="text/xsl" href="' + stylepath( dir, 'albumbild.xsl' ) + '" ?>\n' )
    xmlfile.write( '<albumbild>\n' )
    
    if album_info['title'] != '' : xmlfile.write( '<title>' + album_info['title'] + '</title>\n' ) 
    if album_info['date']  != '' : xmlfile.write( '<date>'  + album_info['date']  + '</date>\n' )

    if prev != None: xmlfile.write( '<navprev>' + prev + '</navprev>\n' )
    if next != None: xmlfile.write( '<navnext>' + next + '</navnext>\n' )

    name = splitext( basename( img['img'] ) )[0]

    xmlfile.write( '<photo>\n' )
    xmlfile.write( '  <url>' + basename( config.lores( img ) ) + '</url>\n' )
    if copyimg : 
        xmlfile.write( '  <orig>' + join( config.copy_dir, basename( orig ) ) + '</orig>\n' )
    xmlfile.write( '  <name>' + name + '</name>\n' )
    if img['desc'] != '' :
        xmlfile.write( '  <desc>' + img['desc'] + '</desc>\n' )
    xmlfile.write( '</photo>\n' )
    xmlfile.write( '</albumbild>\n' )

    xmlfile.close()

#
# write xml file containing year index structure
#
def write_year ( xml, year, daydb ):
    "write xml file containg calendar for a year and links to existing day entries"

    index   = open( xml, encoding='utf-8', mode='w' )

    index.write( '<?xml version="1.0" encoding="UTF8"?>\n' )
    index.write( '<?xml-stylesheet type="text/xsl" href="' + stylepath( '.', 'calendar.xsl' ) + '" ?>\n' )
    index.write( '<year name="%d">\n' % ( year ) )

    # add links previous and next year
    if exists( join( config.top_dir, str(year-1), 'index.html' ) ):
        index.write( '  <navprev>' + join( '..', str(year-1), '' ) + '</navprev>\n' )
    if exists( join( config.top_dir, str(year+1), 'index.html' ) ):
        index.write( '  <navnext>' + join( '..', str(year+1), '' ) + '</navnext>\n' )

    # write months of the year, e.g. actual calendar
    for month in range(1,13):
        index.write( '  <month name="%s">\n' % ( monthnames[ month ] ) )
        index.write( '    <week number="%d">\n' % ( weeknr ) )

        # empty days until first day in month
        date = datetime( year, month, 1 )
        for d in range(1,date.weekday()+1):
            index.write( '      <day></day>\n' )

        for day in range(1,32):
            try:
                date = datetime( year, month, day )
                weekday = date.weekday()
                if date.day in daydb[month]:
                    link = join( '%02d-%02d' % ( month, date.day ), '' )
                    index.write( '      <day><a href="' + link + '">%d</a></day>\n' % date.day )
                else:
                    index.write( '      <day>%d</day>\n' % ( date.day ) )

                if weekday == 6:
                    index.write( '    </week>\n' )
                    weeknr = weeknr+1
                    index.write( '    <week number="%d">\n' % ( weeknr ) )
            except:
                pass

        # and empty days from last day in month
        for d in range(date.weekday()+1,7):
            index.write( '      <day></day>\n' )

        index.write( '    </week>\n' )

        index.write( '  </month>\n' )

    index.write( '</year>\n' )

    index.close()
